/*
 * OPTLAB - Copyright (C) 2018 E. Kuci, C. Geuzaine and P. Duysinx, University of Liege
 *
 * See the LICENSE.txt file for license information. Please report all
 * bugs and problems to the public mailing list <optlab@onelab.info>.
 */

#ifndef _OPTLABC_H_
#define _OPTLABC_H_

/*
 * This file defines the OPTLAB C API (v1.0).
 *
 * Do not edit it directly: it is automatically generated by `api/gen.py'.
 *
 * By design, the OPTLAB C API is purely functional, and only uses elementary
 * types. See `demos/api' for examples.
 */

#include <stddef.h>

#define OPTLAB_API_VERSION "1.0"

#if defined(OPTLAB_DLL)
#if defined(OPTLAB_DLL_EXPORT)
#define OPTLAB_API __declspec(dllexport)
#else
#define OPTLAB_API __declspec(dllimport)
#endif
#else
#define OPTLAB_API
#endif

OPTLAB_API void optlabFree(void *p);
OPTLAB_API void *optlabMalloc(size_t n);

/* Initialize SCP. */
OPTLAB_API void optlabInitialize(const char * type,
                                 int * ierr);

/* Solve the optimization problem. */
OPTLAB_API void optlabRun(int * ierr);

/* Add the objective function. */
OPTLAB_API void optlabAddObjective(const void * func,
                                   int * ierr);

/* Solve the optimization problem. */
OPTLAB_API void optlabAddConstraint(int * ierr);

/* Solve the optimization problem. */
OPTLAB_API void optlabAddGroupOfConstraints(int * ierr);

/* Solve the optimization problem. */
OPTLAB_API void optlabAddVariable(int * ierr);

/* Solve the optimization problem. */
OPTLAB_API void optlabAddGroupOfVariables(int * ierr);

/* Compute the objective function. */
OPTLAB_API void optlabComputeObjective(int * ierr);

/* Finalize Optlab. */
OPTLAB_API void optlabFinalize(int * ierr);

/* Set a numerical option to `value'. `name'. Available categories and options
 * are listed in the Scp reference manual. */
OPTLAB_API void optlabOptionSetNumber(const char * name,
                                      const double value,
                                      int * ierr);

/* Get the `value' of a numerical option. */
OPTLAB_API void optlabOptionGetNumber(const char * name,
                                      double * value,
                                      int * ierr);

/* Print the numerical options. */
OPTLAB_API void optlabOptionShow(int * ierr);

/* Initialize MMA. `numberOfConstraints' ... FIXME TODO */
OPTLAB_API void optlabMmaInitialize(double * currentPoint, size_t currentPoint_n,
                                    double * lowerBound, size_t lowerBound_n,
                                    double * upperBound, size_t upperBound_n,
                                    const int restart,
                                    int * ierr);

/* Get current point. */
OPTLAB_API void optlabMmaGetCurrentPoint(double ** currentPoint, size_t * currentPoint_n,
                                         int * ierr);

/* Update current point (based on first order derivatives and possibly the
 * diagonal entries of the objective function hessian). */
OPTLAB_API void optlabMmaUpdateCurrentPoint(double * constraints, size_t constraints_n,
                                            double * gradientOfObjective, size_t gradientOfObjective_n,
                                            double * gradientOfConstraints, size_t gradientOfConstraints_n,
                                            double * diagHessianOfObjective, size_t diagHessianOfObjective_n,
                                            int * ierr);

/* Get outer iteration. */
OPTLAB_API void optlabMmaGetOuterIteration(int * iteration,
                                           int * ierr);

/* Get the infinite norm between two successive designs. */
OPTLAB_API void optlabMmaGetDesignChange(double * change,
                                         int * ierr);

/* Set the outer move limits. */
OPTLAB_API void optlabMmaSetMoveLimits(double * lowerBound, size_t lowerBound_n,
                                       double * upperBound, size_t upperBound_n,
                                       const double move,
                                       int * ierr);

/* Finalize MMA. */
OPTLAB_API void optlabMmaFinalize(int * ierr);

/* Set a numerical option to `value'. `name'. Available categories and options
 * are listed in the Scp reference manual. */
OPTLAB_API void optlabMmaOptionSetNumber(const char * name,
                                         const double value,
                                         int * ierr);

/* Get the `value' of a numerical option. */
OPTLAB_API void optlabMmaOptionGetNumber(const char * name,
                                         double * value,
                                         int * ierr);

/* Print the numerical options. */
OPTLAB_API void optlabMmaOptionShow(int * ierr);

#endif
